#include "BigInteger.h"
#include <NTL/zz.h>

// BigInteger default constructor that sets the value to zero
BigInteger::BigInteger()  {
	
	bigInt = 0;
}

// BigInteger constructor that accepts a BigInteger value
BigInteger::BigInteger(ZZ v)  {
	
	bigInt = v;
}

// BigInteger constructor that accepts a long value parameter
BigInteger::BigInteger(long v)  {

	bigInt = to_ZZ(v);
}

// BigInteger constructor that accepts an unsigned long valye parameter
BigInteger::BigInteger(unsigned long v)  {
	
	bigInt = to_ZZ(v);
}

// BigInteger constructor that accepts an int valye parameter
BigInteger::BigInteger(int v)  {
	
	bigInt = to_ZZ(v);
}

// BigInteger constructor that accepts an unsigned int valye parameter
BigInteger::BigInteger(unsigned int v)  {
	
	bigInt = to_ZZ(v);
}

// BigInteger constructor that accepts a char array parameter
BigInteger::BigInteger(char *v)  {

	bigInt = to_ZZ(v);
}

BigInteger::~BigInteger()  {
	
	bigInt.~ZZ();
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Converts a BigInteger value to long value
long BigInteger::ToLong(const BigInteger& a)  { 
	
	return NTL_ztoint(a.bigInt.rep); 
}

// Converts a BigInteger value to int value
int BigInteger::ToInt(const BigInteger& a)  {
	
	return int(NTL_ztoint(a.bigInt.rep));
}

// Converts a BigInteger value to unsigned long value
unsigned long BigInteger::ToULong(const BigInteger& a)  { 
	
	return NTL_ztouint(a.bigInt.rep); 
}

// Converts a BigInteger value to unsigned int value
unsigned int BigInteger::ToUInt(const BigInteger& a)  { 
	
	return (unsigned int)(NTL_ztouint(a.bigInt.rep));
}

// Converts a BigInteger value to double value
double BigInteger::ToDouble(const BigInteger& a)  { 
	
	return NTL_zdoub(a.bigInt.rep);
}

// Converts a BigInteger value to float value
float BigInteger::ToFloat(const BigInteger& a)  { 
	
	return float(NTL_zdoub(a.bigInt.rep));
}

BigInteger& BigInteger::operator=(const BigInteger& a)  { 
	
	bigInt = a.bigInt;
	
	return *this;
}

BigInteger& BigInteger::operator=(long a)  {
	
	bigInt = a;
	
	return *this; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////

// A function that performs division of two BigInteger numbers, the result will round to the 
// nearest BigInteger
BigInteger Round_divide(BigInteger number, BigInteger divider)  {
	
	BigInteger remainder = number % divider;
	BigInteger result = number / divider;

	if (divider % 2 == 0)  {
		if (remainder >= divider / 2)
			result++;
	}
	else  {
		if (remainder > divider / 2)
			result++;
	}
	return result;
}

long operator==(const BigInteger& a, const BigInteger& b)  {

	return NTL_zcompare(a.bigInt.rep, b.bigInt.rep) == 0;
}

long operator!=(const BigInteger& a, const BigInteger& b)  {
	
	return NTL_zcompare(a.bigInt.rep, b.bigInt.rep) != 0;
}

long operator<(const BigInteger& a, const BigInteger& b)  {
	
	return NTL_zcompare(a.bigInt.rep, b.bigInt.rep) < 0;
}

long operator>(const BigInteger& a, const BigInteger& b)  {
	
	return NTL_zcompare(a.bigInt.rep, b.bigInt.rep) > 0;
}

long operator<=(const BigInteger& a, const BigInteger& b)  {
	
	return NTL_zcompare(a.bigInt.rep, b.bigInt.rep) <= 0;
}

long operator>=(const BigInteger& a, const BigInteger& b)  {
	
	return NTL_zcompare(a.bigInt.rep, b.bigInt.rep) >= 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////

long operator==(const BigInteger& a, long b)  {

	return NTL_zscompare(a.bigInt.rep, b) == 0; 
}

long operator!=(const BigInteger& a, long b)  {
	
	return NTL_zscompare(a.bigInt.rep, b) != 0; 
}

long operator<(const BigInteger& a, long b)  {
	
	return NTL_zscompare(a.bigInt.rep, b) < 0;
}

long operator>(const BigInteger& a, long b)  {
	
	return NTL_zscompare(a.bigInt.rep, b) > 0;
}

long operator<=(const BigInteger& a, long b)  {
	
	return NTL_zscompare(a.bigInt.rep, b) <= 0;
}

long operator>=(const BigInteger& a, long b)  {
	
	return NTL_zscompare(a.bigInt.rep, b) >= 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////

long operator==(long a, const BigInteger& b)  { 
	
	return b == a; 
}

long operator!=(long a, const BigInteger& b)  { 
	
	return b != a; 
}

long operator<(long a, const BigInteger& b)  { 
	
	return b > a; 
}

long operator>(long a, const BigInteger& b)  { 
	
	return b < a; 
}

long operator<=(long a, const BigInteger& b)  { 
	
	return b >= a; 
}

long operator>=(long a, const BigInteger& b)  { 
	
	return b <= a; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////

BigInteger operator+(const BigInteger& a, const BigInteger& b)  {
	
	ZZ x;

	x = a.bigInt + b.bigInt;
	
	return BigInteger(x); 
}

BigInteger operator+(const BigInteger& a, long b)  {
	
	ZZ x;

	x = a.bigInt + b;
	
	return BigInteger(x); 
}

BigInteger operator+(long a, const BigInteger& b)  {
	
	ZZ x;

	x = a + b.bigInt;
	
	return BigInteger(x); 
}

BigInteger operator-(const BigInteger& a, const BigInteger& b)  {

	ZZ x;

	x = a.bigInt - b.bigInt;

	return BigInteger(x);
}

BigInteger operator-(const BigInteger& a, long b)  {

	ZZ x;

	x = a.bigInt - b;

	return BigInteger(x);
}

BigInteger operator-(long a, const BigInteger& b)  {

	ZZ x;

	x = a - b.bigInt;

	return BigInteger(x);
}

BigInteger operator-(const BigInteger& a)  {

	ZZ x;

	x = -a.bigInt;

	return BigInteger(x);
}

BigInteger abs(const BigInteger& a)  {
	
	ZZ x;

	x = abs(a.bigInt);

	return BigInteger(x);
}

BigInteger& operator+=(BigInteger& x, const BigInteger& a)  { 
	
	x.bigInt += a.bigInt;
	
	return x; 
}

BigInteger& operator+=(BigInteger& x, long a)  { 
	
	x.bigInt += a;

	return x;
}

BigInteger& operator-=(BigInteger& x, const BigInteger& a)  {

	x.bigInt -= a.bigInt;

	return x;
}

BigInteger& operator-=(BigInteger& x, long a)  { 
	
	x.bigInt -= a;

	return x;
}

BigInteger& operator++(BigInteger& x)  {
	
	x.bigInt++;

	return x;
}

void operator++(BigInteger& x, int)  { 
	
	x.bigInt++;
}

BigInteger& operator--(BigInteger& x)  { 
	
	x.bigInt--;

	return x; 
}

void operator--(BigInteger& x, int)  { 
	
	x.bigInt--;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Multiplication
BigInteger operator*(const BigInteger& a, const BigInteger& b)  { 
	
	ZZ x; 
	x = a.bigInt * b.bigInt;

	return BigInteger(x);
}

BigInteger operator*(const BigInteger& a, long b)  { 
	
	ZZ x; 
	x = a.bigInt * b;

	return BigInteger(x); 
}

BigInteger operator*(long a, const BigInteger& b)  { 
	
	ZZ x; 
	x = a * b.bigInt;
	
	return BigInteger(x);
}

BigInteger& operator*=(BigInteger& x, const BigInteger& a)  { 
	
	x.bigInt *= a.bigInt;

	return x;
}

BigInteger& operator*=(BigInteger& x, long a)  { 
	
	x.bigInt *= a;
	
	return x; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Division
BigInteger operator/(const BigInteger& a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a.bigInt / b.bigInt;

	return BigInteger(x);
}

BigInteger operator/(const BigInteger& a, long b)  { 
	
	ZZ x; 

	x = a.bigInt / b;
	
	return BigInteger(x);
}

BigInteger operator%(const BigInteger& a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a.bigInt % b.bigInt;

	return BigInteger(x);
}

long operator%(const BigInteger& a, long b)  { 
	
	long x;

	x = a.bigInt % b;

	return x; 
}

BigInteger& operator/=(BigInteger& x, const BigInteger& b)  { 
	
	x.bigInt /= b.bigInt;

	return x; 
} 

BigInteger& operator/=(BigInteger& x, long b)  { 
	
	x.bigInt /= b;

	return x;
} 

BigInteger& operator%=(BigInteger& x, const BigInteger& b)  { 
	
	x.bigInt %= b.bigInt;
	
	return x; 
} 

//////////////////////////////////////////////////////////////////////////////////////////////////
// Shift operators
BigInteger operator>>(const BigInteger& a, long n)  { 
	
	ZZ x; 

	x = a.bigInt >> n;

	return BigInteger(x);
}

BigInteger operator<<(const BigInteger& a, long n)  { 
	
	ZZ x; 
	
	x = a.bigInt << n;
	
	return BigInteger(x);
}

BigInteger& operator<<=(BigInteger& x, long n)  { 
	
	x.bigInt <<= n;
    
	return x; 
}

BigInteger& operator>>=(BigInteger& x, long n)  { 
	
	x.bigInt >>= n;
	
	return x; 
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Bitwise operators
BigInteger operator&(const BigInteger& a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a.bigInt & b.bigInt;

	return BigInteger(x);
}

BigInteger operator&(const BigInteger& a, long b)  { 
	
	ZZ x; 
	
	x = a.bigInt & b;

	return BigInteger(x);
}

BigInteger operator&(long a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a & b.bigInt;

	return BigInteger(x); 
}

BigInteger operator|(const BigInteger& a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a.bigInt | b.bigInt;
	
	return BigInteger(x);
}

BigInteger operator|(const BigInteger& a, long b)  { 
	
	ZZ x;
	
	x = a.bigInt | b;

	return BigInteger(x);
}

BigInteger operator|(long a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a | b.bigInt;

	return BigInteger(x);
}

BigInteger operator^(const BigInteger& a, const BigInteger& b)  { 
	
	ZZ x; 

	x = a.bigInt ^ b.bigInt;

	return BigInteger(x);
}

BigInteger operator^(const BigInteger& a, long b)  { 
	
	ZZ x; 
	
	x = a.bigInt ^ b;
		
	return BigInteger(x);
}

BigInteger operator^(long a, const BigInteger& b)  { 
	
	ZZ x; 
	
	x = a ^ b.bigInt;
	
	return BigInteger(x);
}

BigInteger& operator&=(BigInteger& x, const BigInteger& b)  { 
	
	x.bigInt &= b.bigInt;
	
	return x; 
}

BigInteger& operator&=(BigInteger& x, long b)  { 
	
	x.bigInt &= b;
	
	return x; 
}

BigInteger& operator|=(BigInteger& x, const BigInteger& b)  { 
	
	x.bigInt |= b.bigInt;

	return x; 
}

BigInteger& operator|=(BigInteger& x, long b)  { 
	
	x.bigInt |= b;
	
	return x; 
}

BigInteger& operator^=(BigInteger& x, const BigInteger& b)  { 
	
	x.bigInt ^= b.bigInt;
	return x; 
}

BigInteger& operator^=(BigInteger& x, long b)  { 
	
	x.bigInt ^= b;
	
	return x;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Input and output stream
istream& operator>>(istream& s, BigInteger& x)  {
	
	s >> x.bigInt;
	return s;
}

ostream& operator<<(ostream& s, const BigInteger& a)  {

	s << a.bigInt;
	return s;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Square root function
// Square root
BigInteger SqrRoot(const BigInteger& a)  {
	
	BigInteger x(SqrRoot(a.bigInt));

	return x;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// Exponentiation function
// Power
BigInteger Power(const BigInteger& a, long e)  {
	
	ZZ value = power(a.bigInt, e);
	BigInteger x(value);
	
	return x;
}

