#ifndef BIGINTEGER_H
#define BIGINTEGER_H

#include <iostream.h>
#include <NTL/zz.h> 

class BigInteger  {
	
	public:
		ZZ bigInt;
		BigInteger();
		BigInteger(ZZ v);
		BigInteger(long v);
		BigInteger(unsigned long v);
		BigInteger(int v);
		BigInteger(unsigned int v);
		BigInteger(char *v);
		~BigInteger();
		
		long ToLong(const BigInteger& a);
		int ToInt(const BigInteger& a);
		unsigned long ToULong(const BigInteger& a);
		unsigned int ToUInt(const BigInteger& a);
		double ToDouble(const BigInteger& a);
		float ToFloat(const BigInteger& a);
		BigInteger Round_divide(BigInteger number, BigInteger divider);
			
		BigInteger& operator=(const BigInteger& a);
		BigInteger& operator=(long a);

	private:
		
};

BigInteger Round_divide(BigInteger number, BigInteger divider);

long operator==(const BigInteger& a, const BigInteger& b);
long operator!=(const BigInteger& a, const BigInteger& b);
long operator<(const BigInteger& a, const BigInteger& b);
long operator>(const BigInteger& a, const BigInteger& b);
long operator<=(const BigInteger& a, const BigInteger& b);
long operator>=(const BigInteger& a, const BigInteger& b);

long operator==(const BigInteger& a, long b);
long operator!=(const BigInteger& a, long b);
long operator<(const BigInteger& a, long b);
long operator>(const BigInteger& a, long b);
long operator<=(const BigInteger& a, long b);
long operator>=(const BigInteger& a, long b);

long operator==(long a, const BigInteger& b);
long operator!=(long a, const BigInteger& b);
long operator<(long a, const BigInteger& b);
long operator>(long a, const BigInteger& b);
long operator<=(long a, const BigInteger& b);
long operator>=(long a, const BigInteger& b);

// Addition and substraction
BigInteger operator+(const BigInteger& a, const BigInteger& b);
BigInteger operator+(const BigInteger& a, long b);
BigInteger operator+(long a, const BigInteger& b);

BigInteger operator-(const BigInteger& a, const BigInteger& b);
BigInteger operator-(const BigInteger& a, long b);
BigInteger operator-(long a, const BigInteger& b);
BigInteger operator-(const BigInteger& a);

BigInteger abs(const BigInteger& a);

BigInteger& operator+=(BigInteger& x, const BigInteger& a);
BigInteger& operator+=(BigInteger& x, long a);
BigInteger& operator-=(BigInteger& x, const BigInteger& a);
BigInteger& operator-=(BigInteger& x, long a);

BigInteger& operator++(BigInteger& x);
void operator++(BigInteger& x, int);
BigInteger& operator--(BigInteger& x);
void operator--(BigInteger& x, int);

BigInteger operator*(const BigInteger& a, const BigInteger& b);
BigInteger operator*(const BigInteger& a, long b);
BigInteger operator*(long a, const BigInteger& b);
BigInteger& operator*=(BigInteger& x, const BigInteger& a);
BigInteger& operator*=(BigInteger& x, long a);

BigInteger operator/(const BigInteger& a, const BigInteger& b) ;
BigInteger operator/(const BigInteger& a, long b);
BigInteger operator%(const BigInteger& a, const BigInteger& b);
long operator%(const BigInteger& a, long b);
BigInteger& operator/=(BigInteger& x, const BigInteger& b);
BigInteger& operator/=(BigInteger& x, long b);
BigInteger& operator%=(BigInteger& x, const BigInteger& b);

BigInteger operator>>(const BigInteger& a, long n);
BigInteger operator<<(const BigInteger& a, long n) ;
BigInteger& operator<<=(BigInteger& x, long n);
BigInteger& operator>>=(BigInteger& x, long n);

BigInteger operator&(const BigInteger& a, const BigInteger& b) ;
BigInteger operator&(const BigInteger& a, long b);
BigInteger operator&(long a, const BigInteger& b);
BigInteger operator|(const BigInteger& a, const BigInteger& b);
BigInteger operator|(const BigInteger& a, long b);
BigInteger operator|(long a, const BigInteger& b);
BigInteger operator^(const BigInteger& a, const BigInteger& b);
BigInteger operator^(const BigInteger& a, long b);
BigInteger operator^(long a, const BigInteger& b);
BigInteger& operator&=(BigInteger& x, const BigInteger& b);
BigInteger& operator&=(BigInteger& x, long b);
BigInteger& operator|=(BigInteger& x, const BigInteger& b);
BigInteger& operator|=(BigInteger& x, long b);
BigInteger& operator^=(BigInteger& x, const BigInteger& b);
BigInteger& operator^=(BigInteger& x, long b);

istream& operator>>(istream& s, BigInteger& x);
ostream& operator<<(ostream& s, const BigInteger& a);

BigInteger SqrRoot(const BigInteger& a);
BigInteger Power(const BigInteger& a, long e);

#endif