#include <NTL/mat_zz.h>
#include <NTL/ZZ.h>
#include "matrix.h"
#include "BigInteger.h"

// Default Matrix class constructor
Matrix::Matrix()  {
	
	x.SetDims(2, 2);
}

// Matrix class constructor that set the row and column of the matrix
Matrix::Matrix(long row, long col)  {
	
    x.SetDims(row, col);
}

// Matrix class destructor
Matrix::~Matrix()  {
	
}

// Set the dimension of the matrix
void Matrix::SetDimensions(long row, long col)  {
	
	x.SetDims(row, col);
}

// Return the number of rows of the matrix
long Matrix::NumberOfRows()  {
	
	return x.NumRows();
}

// Return the number of columns of the matrix
long Matrix::NumberOfCols()  {
	
	return x.NumCols();
}

// Return the tranpose of the matrix
Matrix Matrix::Transpose(const Matrix& A)  {
	
	Matrix c(1,1);

	c.x = transpose(A.x);

	return c;
}

// Find the inverse of the matrix if it exists
Matrix Matrix::Inverse(const Matrix& A)  {
	
	Matrix c(1,1);

	c.x = inv(A.x);

	return c;
}

// Find the determinant of the matrix
BigInteger Matrix::Determinant(const Matrix& A)  {
	
	ZZ value;

	value = determinant(A.x, 0);

	BigInteger c(value);
	
	return c;
}

// Return a diagonal matrix with n rows. Each diagonal position with value d
Matrix Matrix::Diagonal(long n, const BigInteger& d)  {
	
	Matrix c(1,1);

	c.x =  diag(n, d.bigInt);
   
	return c;
}

// Determine whether the matrix is a zero matrix or not
long Matrix::IsZeroMatrix(const Matrix& a)  {
	
	return IsZero(a.x);
}

//////////////////////////////////////////////////////////////////////////////////////////////////

// Return the matrix entry values in BigInteger
BigInteger Matrix::GetMatrixValue(long row, long col)   {

	ZZ temp;

	temp = x[row - 1][col - 1];

	BigInteger c(temp);

	return c;
}

// Set the matrix entry value (input is BigInteger) 
void Matrix::SetMatrixValue(BigInteger value, long row, long col)  {

	x[row - 1][col - 1] = value.bigInt;
}

// Set the matrix entry value (input is long) 
void Matrix::SetMatrixValue(long value, long row, long col)  {

	x[row - 1][col - 1] = value;
}

//////////////////////////////////////////////////////////////////////////////////////////////////

BigInteger DotProduct(Matrix A, long A_row, Matrix B, long B_row)  {
	
	long i;
	long columnNo;
	BigInteger result = 0;

	columnNo = A.NumberOfCols();

	for (i = 1; i <= columnNo; i++)  {
		result += A.GetMatrixValue(A_row, i) * B.GetMatrixValue(B_row, i);
	}
	return result;
}

Matrix operator+(const Matrix& a, const Matrix& b)  {
   
	Matrix c(1,1);

	c.x = a.x + b.x;

	return c;
}

Matrix operator-(const Matrix& a, const Matrix& b)  {
	
	Matrix c(1,1);

	c.x = a.x + b.x;

	return c;
}

Matrix operator-(const Matrix& a)  {
	
	Matrix c(1,1);

	c.x = -a.x;

	return c;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
Matrix operator*(const Matrix& a, const BigInteger& b)  {
	
	Matrix c(1,1);

	c.x = a.x * b.bigInt;

	return c;
}

Matrix operator*(const Matrix& a, long b)  {
	
	Matrix c(1,1);

	c.x = a.x * b;

	return c;
}

Matrix operator*(const BigInteger& a, const Matrix& b)  {
	
	Matrix c(1,1);

	c.x = a.bigInt * b.x;

	return c;
}

Matrix operator*(long a, const Matrix& b)  {
	
	Matrix c(1,1);

	c.x = a * b.x;

	return c;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
Matrix& operator+=(Matrix& x, const Matrix& a)  {
   
	x.x += a.x;

	return x;
}

Matrix& operator-=(Matrix& x, const Matrix& a)  {

	x.x -= a.x;

	return x;
}

Matrix& operator*=(Matrix& x, const Matrix& a)  {
	
	x.x *= a.x;

	return x;
}
Matrix& operator*=(Matrix& x, const BigInteger& a)  {
	
	x.x *= a.bigInt;

	return x;
}

Matrix& operator*=(Matrix& x, long a)  {
	
	x.x *= a;

	return x;
}