#ifndef MATRIX_H
#define MATRIX_H

#include <iostream.h>
#include <NTL/Matrix.h>
#include <NTL/mat_zz.h>
#include "BigInteger.h"

class Matrix  {
	
	public:
		mat_ZZ x;
		Matrix();
		Matrix(long row, long col);
		~Matrix();
		void SetDimensions(long row, long col);
		long NumberOfRows();
		long NumberOfCols();
		Matrix Transpose(const Matrix& A);
		Matrix Inverse(const Matrix& A);
		BigInteger Determinant(const Matrix& A);
		Matrix Diagonal(long n, const BigInteger& d);
		void Clear(mat_ZZ& a);
		long IsZeroMatrix(const Matrix& a);
		BigInteger GetMatrixValue(long row, long col);
		void SetMatrixValue(BigInteger value, long row, long col);
		void SetMatrixValue(long value, long row, long col);
	
	private:	
};

BigInteger DotProduct(Matrix A, long A_row, Matrix B, long B_row);

// Addition and substraction
Matrix operator+(const Matrix& a, const Matrix& b);
Matrix operator-(const Matrix& a, const Matrix& b);
Matrix operator-(const Matrix& a);

// Matrix/scalar multiplication:
Matrix operator*(const Matrix& a, const BigInteger& b);
Matrix operator*(const Matrix& a, long b);
Matrix operator*(const BigInteger& a, const Matrix& b);
Matrix operator*(long a, const Matrix& b);

// Assignment operations
Matrix& operator+=(Matrix& x, const Matrix& a);
Matrix& operator-=(Matrix& x, const Matrix& a);
Matrix& operator*=(Matrix& x, const Matrix& a);
Matrix& operator*=(Matrix& x, const BigInteger& a);
Matrix& operator*=(Matrix& x, long a);


#endif