#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "spectral.h"
#include "BigInteger.h"
#include "matrix.h"
#include "parse.h"
#include <NTL/zz.h>
#include <NTL/mat_zz.h>

Spectral::Spectral(BigInteger M, BigInteger A)  {
	
	m = M;
	a = A;
}

// A function that finds the minimum value of two BigInteger numbers
BigInteger Spectral::minimum(BigInteger a, BigInteger b)  {
	
	BigInteger c;
	if (a <= b) 
		c = a;
	else c = b;

	return c;
}

// A function that finds the gcd of the two long numbers 
unsigned long Spectral::GCD(unsigned long a, unsigned long b)  {

	long temp;
	long remainder;

	// If a is less than b, swap the two values
	if (a < b)	{
		temp = a;
		a = b;
		b = temp;
	}

	do  {
		remainder = a % b;
		a = b;
		b = remainder;
	}
	while (a != 1 && b != 0);
	
	return a;
}

// A function that finds the gcd of the two BigInteger numbers 
BigInteger Spectral::GCD(BigInteger a, BigInteger b)  {

	BigInteger temp;

	// If a is less than b, swap the two values
	if (a < b)	{
		temp = a;
		a = b;
		b = temp;
	}

	do  {
		BigInteger remainder = a % b;
		a = b;
		b = remainder;
	}
	while (a != 1 && b != 0);
	
	return a;
}

// Initialized step of spectral test
void Spectral::Initialize()  {
	
	
	t = 2;
	h = a;
	h_dash = m;
	p = 1;
	p_dash = 0;
	r = a;
	s = 1 + a * a;
}

void Spectral::EuclideanStep()  {
	
	do  {
		q = h_dash / h;
		u = h_dash - q * h;
		v = p_dash - q * p;

		if (u * u + v * v < s)  {
			s = u * u + v * v;
			h_dash = h;
			h = u;
			p_dash = p;
			p = v;
		}
		else break;
	}
	while (true);
}

BigInteger Spectral::ComputeVt_square(int t)  {
	
	BigInteger Vt;

	if (t = 2)  {
		u = u - h;
		v = v - p;

		if (u * u + v * v < s)  {
			s = u * u + v * v;
			h_dash = u;
			p_dash = v;
		}
		Vt = s;
	}
	return Vt;
}

void Spectral::SetUpUAndV()  {
	
	// Set up U matrix
	U.SetMatrixValue(-h, 1, 1);
	U.SetMatrixValue(p, 1, 2);
	U.SetMatrixValue(-h_dash, 2, 1);
	U.SetMatrixValue(p_dash, 2, 2);

	// Set up V matrix
	V.SetMatrixValue(p_dash, 1, 1);
	V.SetMatrixValue(h_dash, 1, 2);
	V.SetMatrixValue(-p, 2, 1);
	V.SetMatrixValue(-h, 2, 2);

	// If p' is greater than 0, minus the V matrix
	if (p_dash > 0)
		V = -V;
}

void Spectral::Advance_t()  {
	
	long i, l;
	BigInteger temp, tempValue; 
	BigInteger tempU_t, tempU_i;
	
	t++;
	r = (a * r) % m;
		
	Matrix newU(t, t);
	Matrix newV(t, t);

	// Copy the old matrix to the new matrix
	for (i = 1; i <= t - 1; i++)  {
		for (l = 1; l <= t - 1; l++)  {
			tempValue = U.GetMatrixValue(i, l);
			newU.SetMatrixValue(tempValue, i, l);
			tempValue = V.GetMatrixValue(i, l);
			newV.SetMatrixValue(tempValue, i, l);
		}
	}

	// Set U_t to the new row (-r, 0, 0, ..., 0, 1) of t elements
	newU.SetMatrixValue(-r, t, 1);
	for (i = 2; i <= t - 1; i++)  {
		newU.SetMatrixValue(0, t, i);
	}
	newU.SetMatrixValue(1, t, t);

	// Set U_it for 1 <= i < t
	for (i = 1; i < t; i++)  {
		newU.SetMatrixValue(0, i, t);
	}

	// Set V_t to the new row (0, 0, 0, ..., 0, m)
	for (i = 1; i <= t - 1; i++)  {
		newV.SetMatrixValue(0, t, i);
	}
	newV.SetMatrixValue(m, t, t);

	// For 1 <= i < t, set q to round(V_i1 * r / m), V_it to V_i1 * r - q * m, and
	// U_t to U_t + q * U_i
	for (i = 1; i < t; i++)  {
		temp = newV.GetMatrixValue(i, 1);
		q = Round_divide(temp * r, m);
		newV.SetMatrixValue(temp * r - q * m, i, t);
		for (l = 1; l <= t; l++)  {
			tempU_t = newU.GetMatrixValue(t, l);
			tempU_i = newU.GetMatrixValue(i, l);
			tempU_t += q * tempU_i;
			newU.SetMatrixValue(tempU_t, t, l);
		}
	}

	// Set s = min(s, U_t dot U_t), k = t, j = 1
	tempValue = DotProduct(newU, t, newU, t);
	s = minimum(s, tempValue);
	k = t;
	j = 1;

	U = newU;
	V = newV;
}

void Spectral::Transform()  {

	long i, l;
	BigInteger tempV_i, tempV_j;
	BigInteger tempU_i, tempU_j;
	BigInteger V_iDotV_j;
	BigInteger V_jDotV_j;
	
	for (i = 1; i <= t; i++)  {
		
		// Find V_i dot V_j and V_j dot V_j
		V_iDotV_j = DotProduct(V, i, V, j);
		V_jDotV_j = DotProduct(V, j, V, j);
		
		if (i != j && 2 * abs(V_iDotV_j) > V_jDotV_j)  {
			// Set q to round (V_i dot V_j / V_j dot V_j)
			q = Round_divide(V_iDotV_j, V_jDotV_j);
			
			// V_i = V_i - q * V_j
			for (l = 1; l <= t; l++)  {
				tempV_i = V.GetMatrixValue(i, l);
				tempV_j = V.GetMatrixValue(j, l);
				tempV_i -= q * tempV_j;
				V.SetMatrixValue(tempV_i, i, l);
			}

			// U_j = U_j + q * U_i
			for (l = 1; l <= t; l++)  {
				tempU_j = U.GetMatrixValue(j, l);
				tempU_i = U.GetMatrixValue(i, l);
				tempU_j += q * tempU_i;
				U.SetMatrixValue(tempU_j, j, l);
			}

			// Set s = min(s, U_j dot U_j), k = j
			BigInteger tempValue = DotProduct(U, j, U, j);
			s = minimum(s, tempValue);
			k = j;
		}
	}
}

void Spectral::Searching()  {
	
	long i, l;
	BigInteger tempValue;
	BigInteger tempY;
	BigInteger tempU_k;
	BigInteger x_kValue, z_kValue;
	Matrix X(1, t);
	Matrix Y(1, t);
	Matrix Z(1, t);

	// Set X = Y = (0, ..., 0)
	for (i = 1; i <= t; i++)  {
		X.SetMatrixValue(0, 1, i);
		Y.SetMatrixValue(0, 1, i);
	}

	// Set k = t
	k = t;

	for (l = 1; l <= t; l++)  {
		tempValue = (DotProduct(V, l, V, l) * s) / (m * m);
		tempValue = SqrRoot(tempValue);
		Z.SetMatrixValue(tempValue, 1, j);
	}

	do	{
		// If x_k != z_k
		x_kValue = X.GetMatrixValue(1, k);
		z_kValue = Z.GetMatrixValue(1, k);
		if (x_kValue != z_kValue)  {
			// Increase x_k by 1
			x_kValue++;
			X.SetMatrixValue(x_kValue, 1, k);

			// Set Y = Y + U_k
			for (l = 1; l <= t; l++)  {
				tempY = Y.GetMatrixValue(1, l);
				tempU_k = U.GetMatrixValue(1, l);
				tempY += tempU_k;
				Y.SetMatrixValue(tempY, 1, l);
			}
		
			do  {
				// k = k + 1
				k++;
				if (k <= t)  {
					tempValue = -Z.GetMatrixValue(1, k);
					X.SetMatrixValue(tempValue, 1, k);
				
					// Set Y = Y - 2 * z_k * U_k
					for (l = 1; l <= t; l++)  {
						tempY = Y.GetMatrixValue(1, l);
						tempU_k = U.GetMatrixValue(1, l);
						tempY = tempY - 2 * Z.GetMatrixValue(1, k) * tempU_k;
						Y.SetMatrixValue(tempY, 1, l);
					}
				}
				else if (k > t)  {
					// Set s = min(s, Y dot Y)
					tempValue = DotProduct(Y, 1, Y, 1);
					s = minimum(s, tempValue);
					break;
				}
			} while (1);
		}
		
		// Set k = k - 1
		k--;

		if (k < 1)  {
			cout << "Square value of V" << t << " is " << s << endl;
			break;
		}
	}
	while (1);
}

void Spectral::SpectralTest(int T)  {
	
	clock_t start, finish;

	Initialize();
	EuclideanStep();

	BigInteger value = ComputeVt_square(2);
	cout << "Square value of V2 is " << value << endl;

	SetUpUAndV();
	start = clock();

	while (t < T)  {	
		Advance_t();
		do  {
			Transform();
			if (j == t)
				j = 1;
			else j++;

			if (j == k)
				break;
		}
		while (1);
		Searching();
	};
	finish = clock();
	printf("Time taken is %.2f milliseconds to execute\n",((double) (finish - start)) * 1000 / CLK_TCK);
}

int main(int argc, char * argv[])  {
	
	if (argc != 4)   {
        cout << "Usage :" << endl;
		cout << "spectral [a] [m] [T]" << endl;
		cout << "m and a must be in the form of x, x_y or x_y+z" << endl;
		cout << "T must be greater than or equal to 2" << endl;
        return -1;
	}

	BigInteger a = parseInput(argv[1]);
	BigInteger m = parseInput(argv[2]);
	long T = atol(argv[3]);

	if (a == NULL || m == NULL || T < 2)  {
		cout << "Invalid arguments" << endl;
		cout << "m and a must be in the form of x, x_y or x_y+z" << endl;
		cout << "T must be greater than or equal to 2" << endl;
        return -1; 
	}
	
	Spectral s(m, a);

	cout << "a = " << a << endl;
	cout << "m = " << m << endl;
	
	s.SpectralTest(T);

	return 0;
}